--[[
			Select Favorite Mounts

	This stand-alone addon allows you to set mount favorites the same way you do
	battle pet favorites: right-click the mount in the mount journal and Set or
	Remove Favorite status.

	If you use version 2.0.12 or higher of the addon Select, then you can also
	use /select mount:favorite as a filter.

	The default behavior of right-clicking a mount in the mount journal is to
	mount/dismount that mount.  Since right-click now pops out a menu, you can
	double-left click to mount/dismount instead.

	09/11/2013 1.0.4 toc update for 5.4
  05/21/2013 1.0.3 toc update for 5.3
	05/13/2013 1.0.2 fix for stars sometimes not appearing ontop of icons
	05/07/2013 1.0.1 fix for stars on empty slots during searches
	05/05/2013 1.0.0 initial release
]]


SelectFavoriteMountsSaved = {} -- savedvariable
local saved

local sfm = CreateFrame("Frame","SelectFavoriteMounts")
sfm.mounts = {} -- real mount indexes sorted with favorites first, then alphabetically

sfm:SetScript("OnEvent",function(self,event,...)
	if self[event] then
		self[event](self,...)
	end
end)
sfm:RegisterEvent("PLAYER_LOGIN")

function sfm:PLAYER_LOGIN()
	-- unsecure hooks of the three companion functions that deal with an index.
	-- uses our own mount index for second paramter (or passed index if not a mount)
	self.hooks = {}
	for _,funcName in pairs({"GetCompanionInfo","CallCompanion","PickupCompanion"}) do
		self.hooks[funcName] = _G[funcName]
		_G[funcName] = function(companionType,companionIndex,...)
			if companionType and companionType:upper()=="MOUNT" then
				return sfm.hooks[funcName](companionType,sfm.mounts[companionIndex],...)
			else
				return sfm.hooks[funcName](companionType,companionIndex,...)
			end
		end
	end
	saved = SelectFavoriteMountsSaved
	self:UpdateMounts()
	-- there's a chance another addon loaded pet journal already, check for that
	if IsAddOnLoaded("Blizzard_PetJournal") then
		self:ModMountJournal()
	else
		self:RegisterEvent("ADDON_LOADED") -- typically, wait until it's loaded
	end
	self:RegisterEvent("COMPANION_LEARNED")
end

function sfm:COMPANION_LEARNED(...)
	self:UpdateMounts()
end

function sfm:UpdateMounts()
	local numMounts = GetNumCompanions("mount")
	if numMounts ~= self.numKnownMounts then
		-- new mount learned (or just logged in)
		wipe(self.mounts)
		for i=1,numMounts do
			table.insert(self.mounts,i) -- start with an ordered list
		end
		self.numKnownMounts = numMounts
	end
	-- sort list by favorites first, then mount name
	table.sort(self.mounts,function(e1,e2)
		-- get the name of the original index's mount
		local name1 = select(2,self.hooks.GetCompanionInfo("MOUNT",e1))
		local name2 = select(2,self.hooks.GetCompanionInfo("MOUNT",e2))
		local favorite1 = saved[name1]
		local favorite2 = saved[name2]
		if favorite1 and not favorite2 then
			return true
		elseif not favorite1 and favorite2 then
			return false
		elseif name1 and name2 then
			return name1 < name2
		end
	end)
end

function sfm:IsFavorite(name)
	if saved and saved[name] then
		return 1
	end
end

--[[ Changes to default Mount Journal ]]

function sfm:ADDON_LOADED(addon)
	if addon=="Blizzard_PetJournal" then
		self:UnregisterEvent("ADDON_LOADED")
		self:ModMountJournal()
	end
end

function sfm:ModMountJournal()
	for i=1,12 do
		local button = _G["MountJournalListScrollFrameButton"..i]

		-- unsecure hooks of these clicks because we don't want default right-click behavior
		self.hooks.MountJournalListOnClick = button:GetScript("OnClick")
		button:SetScript("OnClick",sfm.MountJournalListOnClick)
		self.hooks.MountJournalDragOnClick = button.DragButton:GetScript("OnClick")
		button.DragButton:SetScript("OnClick",sfm.MountJournalDragOnClick)
		-- post-hook ok for doubleclick
		button:HookScript("OnDoubleClick",sfm.MountJournalDoubleClick)
		button.favorite = button.DragButton:CreateTexture(nil,"OVERLAY")
		button.favorite:SetSize(25,25)
		button.favorite:SetPoint("TOPLEFT",button.DragButton,"TOPLEFT",-8,8)
		button.favorite:SetTexture("Interface\\PetBattles\\PetJournal")
		button.favorite:SetTexCoord(0.11328125,0.16210938,0.02246094,0.04687500)
	end
	sfm.easyMenu = {{notCheckable=true},{notCheckable=true},{notCheckable=true,text=CANCEL}}

	hooksecurefunc("MountJournal_UpdateMountList",sfm.UpdateStars)
	MountJournalListScrollFrameScrollBar:HookScript("OnValueChanged",sfm.UpdateStars)
end

function sfm:UpdateStars()
	if MountJournal:IsVisible() then
		for i=1,12 do
			local button = _G["MountJournalListScrollFrameButton"..i]
			button.favorite:Hide()
			if button.index then
				local _,name = GetCompanionInfo("MOUNT",button.index)
				if sfm:IsFavorite(name) then
					button.favorite:Show()
				end
			end
		end
	end
end

function sfm:MountJournalDoubleClick(button)
	if button=="LeftButton" then
		if self.active then
			DismissCompanion("MOUNT")
		else
			CallCompanion("MOUNT",self.index)
		end
	end
end

function sfm:MountJournalListOnClick(button)
	if button~="LeftButton" then
		MountListItem_OnClick(self,"LeftButton")
		sfm.PopoutMenu(self)
	else
		sfm:CloseMenu()
		sfm.hooks.MountJournalListOnClick(self,button)
	end
end

function sfm:MountJournalDragOnClick(button)
	if button~="LeftButton" then
		MountListItem_OnClick(self:GetParent(),"LeftButton")
		sfm.PopoutMenu(self:GetParent())
	else
		sfm:CloseMenu()
		sfm.hooks.MountJournalDragOnClick(self,button)
	end
end

function sfm:CloseMenu()
	if UIDropDownMenu_GetCurrentDropDown()==SelectFavoriteMountsMenu then
		CloseDropDownMenus()
	end
end

function sfm:PopoutMenu()
	local _,name = GetCompanionInfo("MOUNT",self.index)
	sfm.easyMenu[1].arg1 = self.index
	sfm.easyMenu[2].arg1 = name
	if self.active then -- if mounted, 'Dismount' menu item
		sfm.easyMenu[1].text = BINDING_NAME_DISMOUNT
		sfm.easyMenu[1].func = sfm.Dismount
	else -- otherwise, 'Mount' menu item
		sfm.easyMenu[1].text = MOUNT
		sfm.easyMenu[1].func = sfm.Mount
	end
	if saved[name] then -- if favorite, 'Remove Favorite' menu item
		sfm.easyMenu[2].text = BATTLE_PET_UNFAVORITE
		sfm.easyMenu[2].func = sfm.RemoveFavorite
	else -- otherwise, 'Set Favorite' menu item
		sfm.easyMenu[2].text = BATTLE_PET_FAVORITE
		sfm.easyMenu[2].func = sfm.AddFavorite
	end
	local menuFrame = SelectFavoriteMountsMenu or CreateFrame("Frame","SelectFavoriteMountsMenu",MountJournal,"UIDropDownMenuTemplate")
	EasyMenu(sfm.easyMenu,menuFrame,"cursor",0,0,"MENU")
end

function sfm:AddFavorite(name)
	saved[name] = 1
	sfm:UpdateMounts()
	MountJournal_DirtyList(MountJournal)
	MountJournal_UpdateMountList(MountJournal)
end

function sfm:RemoveFavorite(name)
	saved[name] = nil
	sfm:UpdateMounts()
	MountJournal_DirtyList(MountJournal)
	MountJournal_UpdateMountList(MountJournal)
end

function sfm:Mount(index)
	CallCompanion("MOUNT",index)
end

function sfm:Dismount()
	DismissCompanion("MOUNT")
end
